;	$Id: approx.lsp,v 1.2 1999/08/06 13:46:18 pawel Exp $	
;;; package APPROX
;;; implementation of function approximators
;;; Copyleft 1999 Pawel Cichosz

(require 'util)
(use-package "UTIL")

(defvar *weights* '(0.0 0.0 0.0))
(defparameter *beta* 0.1)
(defparameter *beta-decay* 1.0)
(defparameter *online* t)
(defparameter *random* nil)
(defparameter *niter* 100)
(defparameter *trace* t)

(defun instance-features (x)
  (append (first x) '(1)))

(defun instance-target (x)
  (second x))

(defun linear-value (weights x)
  (reduce #'+
          (mapcar #'*
                  weights
                  (instance-features x))))

(defun linear-approximator (instances &key
                                      (weights *weights*)
                                      (niter *niter*))
  (dotimes (i niter (setq *weights* weights))
    (let ((inst (if *random*
                    (random-shuffle instances)
                    instances))
          (msq-error 0.0)
          (delta (make-list (length weights) :initial-element 0.0)))
      (dolist (x inst)
        (let* ((output (linear-value weights x))
               (target (instance-target x))
               (error1 (- target output))
               (delta1 (mapcar #'(lambda (w f)
                                   (* *beta* error1 f))
                               weights
                               (instance-features x))))
          (incf msq-error (/ (* error1 error1) (length inst)))
          (if *online*
              (map-into weights #'+ weights delta1)
              (map-into delta #'+ delta delta1))))
      (unless *online*
        (map-into weights #'+ weights delta))
      (setq *beta* (* *beta* *beta-decay*))
      (when *trace*
        (format t "~&Iteration ~A:~T~A" i msq-error)))))

