;;;	$Id: info.lsp,v 1.15 1998/02/20 17:24:45 pawel Exp $
;;; package INFO
;;; some information-theoretic functions
;;; Copyleft 1998 Pawel Cichosz

(provide 'info)
(require 'concept)

(defpackage "INFO"
    (:documentation "Package providing some basic information-theoretic
functions")
  (:use "COMMON-LISP" "CONCEPT")
  (:export "ENTROPY"
           "INFORMATION"
           "INFO"))

(in-package "INFO")


(declaim (ftype (function (fixnum list) float) entropy))
(defun entropy (n instances)
  "Compute the entropy of attribute number n for instances."
  (let ((e 0.0)                         ; computed entropy
	(ni (length instances))         ; number of instances
	(a (attribute-at n)))              ; the attribute of interest
    (do-attribute-values (v a e)        ; for all values v
      (let* ((inv (instances-select-v instances n v))
             (w (/ (length inv) ni)))   ; weigth given to v
        (incf e (* w (information inv)))))))


(declaim (inline information)
         (ftype (function (list) float) information))
(defun information (instances)
  "Compute the information for instances."
  (if (endp instances)                  ; if no instances
      most-positive-single-float        ; assume maximum information (useless)
      (let ((e 0.0)                     ; computed information
            (ni (length instances)))    ; number of instances
        (do-categories (c e)            ; for all categories
          (incf e (info (/ (instances-count-c instances c) ni)))))))


(declaim (inline info)
         (ftype (function (real (0 1)) float) info))
(defun info (p)
  "Compute the information resulting from an event with probability p."
  (if (= p 0.0)
      0.0
      (- (* p (log p 2)))))
